<?php
# It is the context of the subscription, in which a customer can manage its VPSes.
# It must correspond to a tenant created for the subscriber in the remote application system.
if (!defined('APS_DEVELOPMENT_MODE')) define('APS_DEVELOPMENT_MODE', 'on');

require_once "aps/2/runtime.php";

/**
 * Class context
 * @type("http://aps-standard.org/samples/vrd/context/1.0")
 * @implements("http://aps-standard.org/types/core/subscription/service/1.0","http://aps-standard.org/types/core/rated-data-support/1.0")
 */
class context extends \APS\ResourceBase
{
## Strong relation (link) to the application instance
    /**
     * @link("http://aps-standard.org/samples/vrd/cloud/1.0")
     * @required
     */
    public $cloud;

## Weak relation (link) to collection of VPSes
    /**
     * @link("http://aps-standard.org/samples/vrd/vps/1.0[]")
     */
    public $vpses;

    public function provision()
    {
        ## Create a folder for the account to simulate a cloud application
        $accountId = $this->account->aps->id;
        if (!file_exists($accountId)) {
            mkdir($accountId);
        }
    }

    public function unprovision()
    {
        ## Remove the folder of the account to simulate a cloud application
        $accountId = $this->account->aps->id;
        if (file_exists($accountId)) {
            array_map('unlink', glob($accountId."/*.*"));
            rmdir($accountId);
        }
        $deletedDir = "deleted";
        if (file_exists($deletedDir)) {
            $pollRetry = 3;
            $pollCounter = fopen($deletedDir."/".$accountId.".pollCounter", 'w');
            fwrite($pollCounter, $pollRetry);
            fclose($pollCounter);
        }
    }

## The method is inherited from the RatedDataSupport abstract APS type.

    public function exportRatedData($lastInvoiceDate)
    {
        $logger = \APS\LoggerRegistry::get();

        $endTime = mktime();
        $invoiceEndDate = date("Y-m-d\Th:i:s", $endTime);

        ## Determine the cases when the APS resource is "ready" but the response must be "204 No Content":
         # 1. The directory was occasionally deleted.
         # 2. There are no provisioned resources.
        $accountId = $this->account->aps->id;
        if ($this->aps->status != "aps:deleted" and (!file_exists($accountId) or count(glob($accountId."/*.json")) == 0)) {
            # $logger->info("Exception 204 will be thrown: No Content");
            throw new \Rest\NoContent();
        }

        ## Determine the cases when the response must be "204 No Content":
         # 3. The subscription is deleted.
        if ($this->aps->status == "aps:deleted") {
            $fileName = "deleted/".$accountId.".pollCounter";
            $pollCounter = (int)file_get_contents($fileName) or die("Unable to open file! ".$fileName);
            ## Decrement the counter and save it before calling the 204 exception:
            # $logger->info("Poll Counter from the file: ". $pollCounter);
            if ($pollCounter > 1) {
                $pollCounter--;
                $fh = fopen($fileName, 'w');
                fwrite($fh, "$pollCounter");
                fclose($fh);
                throw new \Rest\NoContent();
            }
        }

        # Get a list of all accounts from the "APS-Account-Hierarchy" HTTP header:
        $headers = getallheaders();
        $accounts = explode(", ", $headers["APS-Account-Hierarchy"]);

        $report = array(
            "lastInvoiceDate" => $invoiceEndDate,
            "charges" => []
        );

        # This will accumulate charges for every VPS:
        $charges = array();
        $timeDelta = ($endTime - strtotime($lastInvoiceDate)) / 3600;

        ## The rate to convert monthly price to the price for the time delta:
        $usageRate = $timeDelta / (30 * 24);
        ## Price for every lower reseller and down to the customer will be increased by the following rate (110% in this example):
        $markUpRate = 1.1;

        $charge = array(
            "accountId" => $accounts[0],
            "chargeType" => "CHARGE",
            "currencyCode" => "USD",
            "chargeStartDate" => $lastInvoiceDate,
            "chargeEndDate" => $invoiceEndDate
        );

        if ($this->aps->status == "aps:deleted") {
            $charge["unitOfMeasure"] = "item-h";
            $charge["amount"] = 1 * $usageRate;
            $charge["unitPrice"] = 10;
            $charge["totalCost"] = round($charge["amount"] * $charge["unitPrice"], 2);
            $charge["skuId"] = "VPS-Cloud-After-Subscription-Destroy";
            $charge["description"] = "The subscription is destroyed, but I am still alive.";
            $chargeHash = md5(json_encode($charge));
            $charge["hash"] = $chargeHash;

            array_push($charges, $charge);
        }
        else {
            ## All VPS JSON files created by the customer:
            $files = array_diff(scandir($accountId), array('..', '.'));
            
            foreach ($files as $filename) {
                $fileContent = file_get_contents($accountId . '/' . $filename) or die("Unable to open file!");
                $vps = json_decode($fileContent, true);
                $vpsId = pathinfo($filename)['filename']; // File name w/o extension.

                ## Charge for VPS as a resource:
                $vpsCharge = $charge;
                $vpsCharge["unitOfMeasure"] = "item-h";
                $vpsCharge["amount"] = 1 * $usageRate;  # Always one VPS here
                $vpsCharge["unitPrice"] = 12.5; # Fee for 1 VPS per month
                $vpsCharge["totalCost"] = round($vpsCharge["amount"] * $vpsCharge["unitPrice"], 2);
                $vpsCharge["skuId"] = "VPS - Container";
                $vpsCharge["description"] = "Virtual private server - Container variation";
                $vpsHash = md5(json_encode($vpsCharge));
                $vpsCharge["hash"] = $vpsHash;

                ## Charge for CPU cores:
                $cpuCharge = $charge;
                $cpuCharge["unitOfMeasure"] = "item-h";
                ## Account CPU for active VPS only:
                $cpuCharge["amount"] = ($vps["state"] == "Running") ? $vps["hardware"]["CPU"]["number"] * $usageRate : 0;
                $cpuCharge["unitPrice"] = 4.5; # Fee for 1 CPU core per month
                $cpuCharge["totalCost"] = round($cpuCharge["amount"] * $cpuCharge["unitPrice"], 2);
                $cpuCharge["skuId"] = "VPS - CPU, cores";
                $cpuCharge["description"] = "Virtual private server - CPU power";
                $cpuHash = md5(json_encode($cpuCharge));
                $cpuCharge["hash"] = $cpuHash;

                ## Charge for RAM MB-h:
                $ramCharge = $charge;
                $ramCharge["unitOfMeasure"] = "MB-h";
                ## Account RAM for active VPS only:
                $ramCharge["amount"] = ($vps["state"] == "Running") ? $vps["hardware"]["memory"] * $usageRate : 0;
                $ramCharge["unitPrice"] = 0.15; # Fee for 1 MB of RAM per month
                $ramCharge["totalCost"] = round($ramCharge["amount"] * $ramCharge["unitPrice"], 2);
                $ramCharge["skuId"] = "VPS - RAM, MB-h";
                $ramCharge["description"] = "Virtual private server - Memory space";
                $ramHash = md5(json_encode($ramCharge));
                $ramCharge["hash"] = $ramHash;

                ## Charge for storage GB-h:
                $diskCharge = $charge;
                $diskCharge["unitOfMeasure"] = "GB-h";
                $diskCharge["amount"] = $vps["hardware"]["diskspace"] * $usageRate;
                $diskCharge["unitPrice"] = 0.55; # Fee for 1 GB of disk space per month
                $diskCharge["totalCost"] = round($diskCharge["amount"] * $diskCharge["unitPrice"], 2);
                $diskCharge["skuId"] = "VPS - Storage, GB-h";
                $diskCharge["description"] = "Virtual private server - Disk space";
                $diskHash = md5(json_encode($diskCharge));
                $diskCharge["hash"] = $diskHash;

                array_push($charges, $vpsCharge, $cpuCharge, $ramCharge, $diskCharge);
            }
        }

        $report["charges"] = $charges;
        $markUp = $markUpRate;
        for ($num = 1; $num < count($accounts); $num++) {
            $accountId = $accounts[$num];
            foreach ($charges as $charge) {
                $newcharge = $charge;
                $newcharge["accountId"] = $accountId;
                $newcharge["unitPrice"] = $newcharge["unitPrice"] * $markUp;
                $newcharge["totalCost"] = round($newcharge["unitPrice"] * $newcharge["amount"], 2);
                $hash = md5(json_encode($newcharge));
                $newcharge["hash"] = $hash;

                array_push($report["charges"], $newcharge);
            }
            $markUp = $markUp * $markUpRate;
        }
        return $report;
    }
}

